@@ -4,17 +4,17 @@ package Business::DK::CPR;
 
 use strict;
 use warnings;
-use vars qw($VERSION @EXPORT_OK);
 use Carp qw(croak carp);
-use Business::DK::CVR qw(_length _calculate_sum);
-use Business::DK::PO qw(_argument _content);
+use Business::DK::CVR qw(_calculate_sum);
 use Date::Calc qw(check_date);
 use base 'Exporter';
 use integer;
 use Tie::IxHash;
+use Readonly;
+use Params::Validate qw( validate_pos SCALAR );
 
-$VERSION   = '0.06';
-@EXPORT_OK = qw(
+our $VERSION   = '0.07';
+our @EXPORT_OK = qw(
     validate
     validateCPR
     generate
@@ -23,12 +23,12 @@ $VERSION   = '0.06';
     validate2007
     generate2007
     calculate
-    _checkdate
 );
 
 use constant MODULUS_OPERAND_1968 => 11;
 use constant MODULUS_OPERAND_2007 => 6;
 use constant DATE_LENGTH          => 6;
+use constant CONTROL_CIFER_LENGTH => 4;
 use constant VALID                => 1;
 use constant VALID_MALE           => 1;
 use constant VALID_FEMALE         => 2;
@@ -36,7 +36,7 @@ use constant INVALID              => 0;
 use constant FEMALE               => 'female';
 use constant MALE                 => 'male';
 
-my @controlcifers = qw(4 3 2 7 6 5 4 3 2 1);
+Readonly my @controlcifers => qw(4 3 2 7 6 5 4 3 2 1);
 
 my %female_seeds;
 tie %female_seeds, 'Tie::IxHash',
@@ -63,14 +63,14 @@ sub merge {
 }
 
 sub calculate {
-    my $birthdate = shift;
-
-    _assert_date($birthdate);
+    my ($birthdate) = @_;
+    validate_pos( @_, { type => SCALAR, callbacks => { 'date' => \&_checkdate } } );
 
     my @cprs;
     for ( 1 .. 999 ) {
         my $n = sprintf '%03s', $_;
 
+        #From DK::Business::CVR
         my $sum = _calculate_sum( ( $birthdate . $n ), \@controlcifers );
         my $mod = $sum % MODULUS_OPERAND_1968;
 
@@ -88,25 +88,24 @@ sub calculate {
     }
 }
 
-sub _assert_date {
-    my $birthdate = shift;
-
-    if ( !$birthdate ) {
-        _argument(DATE_LENGTH);
-    }
-    _content($birthdate);
-    _length( $birthdate, DATE_LENGTH );
-    _checkdate($birthdate);
-
-    return VALID;
-}
-
 sub validateCPR {
+    #We postpone parameter validation
     return validate(shift);
 }
 
+sub _length {
+    my ( $number, $length ) = @_;
+
+    if ( length($number) != $length ) {
+        croak "argument: $number has to be $length digits long";
+    }
+    return 1;
+}
+
 sub validate {
-    my $controlnumber = shift;
+    my ($controlnumber) = @_;
+
+    #We postpone parameter validation
 
     my $rv;
     if ( $rv = validate1968($controlnumber) ) {
@@ -117,16 +116,17 @@ sub validate {
 }
 
 sub validate2007 {
-    my $controlnumber = shift;
-
-    _assert_date( substr $controlnumber, 0, DATE_LENGTH );
-
+    my ($controlnumber) = @_;
+    validate_pos( @_, { type => SCALAR, regex => qr/^\d+$/ } );
+    
+    _checkdate( substr $controlnumber, 0, DATE_LENGTH );
     _assert_controlnumber($controlnumber);
 
-    my $control = substr $controlnumber, DATE_LENGTH, 4;
+    my $control = substr $controlnumber, DATE_LENGTH, CONTROL_CIFER_LENGTH;
 
     my $remainder = $control % MODULUS_OPERAND_2007;
 
+    #Our own merge, could be Hash::Merge's merge
     my %seeds = %{ merge( \%male_seeds, \%female_seeds ) };
 
     if ( my $series = $seeds{$remainder} ) {
@@ -155,9 +155,11 @@ sub validate2007 {
 }
 
 sub validate1968 {
-    my $controlnumber = shift;
+    my ($controlnumber) = @_;
 
-    _assert_date( substr $controlnumber, 0, DATE_LENGTH );
+    validate_pos( @_, { type => SCALAR, regex => qr/^\d+$/ } );
+
+    _checkdate( substr $controlnumber, 0, DATE_LENGTH );
     _assert_controlnumber($controlnumber);
 
     my $sum = _calculate_sum( $controlnumber, \@controlcifers );
@@ -176,31 +178,25 @@ sub validate1968 {
 }
 
 sub _is_equal {
-    my $operand = shift;
-
+    my ($operand) = @_;
+    
+    validate_pos( @_, { type => SCALAR, regex => qr/^\d+$/ } );
+    
     return ( not( $operand % 2 ) );
 }
 
 sub _assert_controlnumber {
-    my $controlnumber = shift;
+    my ($controlnumber) = @_;
 
-    my $controlcode_length = scalar @controlcifers;
+    validate_pos( @_, { type => SCALAR, regex => qr/^\d+$/ } );
 
-    if ( not $controlnumber ) {
-        _argument($controlcode_length);
-    }
-    _content($controlnumber);
-    _length( $controlnumber, $controlcode_length );
+    _length( $controlnumber, scalar @controlcifers );
 
     return VALID;
 }
 
 sub _checkdate {
-    my $birthdate = shift;
-
-    if ( not $birthdate ) {
-        croak 'argument for birthdate should be provided';
-    }
+    my $birthdate = $_[0];
 
     if (not($birthdate =~ m{\A #beginning of line
               (\d{2}) #day of month, 2 digit representation, 01-31
@@ -223,7 +219,12 @@ sub _checkdate {
 
 sub generate {
     my ( $birthdate, $gender ) = @_;
-
+    
+    validate_pos( @_,
+        { type => SCALAR, callbacks => { 'date' => \&_checkdate }, },
+        { type => SCALAR, optional => 1 },
+    );
+    
     my %cprs;
 
     my @cprs1968 = generate1968( $birthdate, $gender );
@@ -240,10 +241,13 @@ sub generate {
 }
 
 sub generate2007 {
-    my $birthdate = shift;
-    my $gender = shift || undef;
+    my ( $birthdate, $gender ) = @_;
 
-    _assert_date($birthdate);
+    #TODO assert gender?
+    validate_pos( @_,
+        { type => SCALAR, callbacks => { 'date' => \&_checkdate }, },
+        { type => SCALAR, optional => 1 },
+    );
 
     my @cprs;
     my %seeds;
@@ -279,11 +283,14 @@ sub generate2007 {
 }
 
 sub generate1968 {
-    my $birthdate = shift;
-    my $gender = shift || undef;
-
-    _assert_date($birthdate);
+    my ( $birthdate, $gender ) = @_;
 
+    #TODO assert gender?
+    validate_pos( @_,
+        { type => SCALAR, callbacks => { 'date' => \&_checkdate }, },
+        { type => SCALAR, optional => 1 },
+    );
+    
     my @cprs;
     my @malecprs;
     my @femalecprs;
@@ -333,7 +340,7 @@ Business::DK::CPR - Danish CPR code generator/validator
 
 =head1 VERSION
 
-This documentation describes version 0.06
+This documentation describes version 0.07
 
 =head1 SYNOPSIS
 
@@ -400,29 +407,29 @@ argument and returns:
 =back
 
 It dies if the CPR number is malformed or in any way unparsable, be aware that
-the 6 first digits are representing a date (SEE: L</_checkdate> function below).
+the 6 first digits are representing a date (SEE: L<_checkdate|/_checkdate> function below).
 
 In brief, the date indicate the person's birthday, the last 4 digits are
 representing a serial number and control cipher.
 
 For a more thorough discussion on the format of CPR numbers please refer to the
-L<SEE ALSO> section.
+L<SEE ALSO|/SEE ALSO> section.
 
-L</validate1968> is the old form of the CPR number. It is validated using
-modulus 11.
+L<validate1968|/validate1968> is the old form of the CPR number. It is validated
+using modulus 11.
 
 The new format introduced in 2001 (put to use in 2007, hence the name used
-throughout this package) can be validated using L</validate2007> and generate
-using L</validate2007>.
+throughout this package) can be validated using L<validate2007|/validate2007> and
+generate using L<generate2007|/generate2007>.
 
-The L</validate> subroutine wraps both validators and checks using against both.
+The L<validate|/validate> subroutine wraps both validators and checks using against both.
 
-The L</generate> subroutine wraps both generators and accumulated the results.
+The L<generate|/generate> subroutine wraps both generators and accumulated the results.
 
 NB! it is possible to make fake CPR numbers that appear valid, please see
 MOTIVATION and the L</calculate> function.
 
-L</validate> is also exported as: L</validateCPR>, which is less imposing.
+L<validate|/validate> is also exported as: L<validateCPR|/validateCPR>, which is less imposing.
 
 =head2 validateCPR
 
@@ -439,7 +446,7 @@ Validation against the CPR algorithm introduced in 2007.
 =head2 generate
 
 This is a wrapper around calculate, so the naming is uniform to
-L<Business::DK::CVR>
+L<Business::DK::CVR|Business::DK::CVR>
 
 This function takes an integer representing a date and calculates valid CPR
 numbers for the specified date. In scalar context returns the number of valid
@@ -449,26 +456,41 @@ If the date is malformed or in any way invalid or unspecified the function dies.
 
 =head2 generate1968
 
-Specialized generator for validate1968 compatible CPR numbers. See: L</generate>
+Specialized generator for L<validate1968|/validate1968> compatible CPR numbers. See: L<generate|/generate>
 
 =head2 generate2007
 
-Specialized generator for validate2007 compatible CPR numbers. See: L</generate>
+Specialized generator for L<validate2007|/validate2007> compatible CPR numbers. See: L<generate|/generate>
 
 =head2 calculate
 
-See L</generate> and L</generate1968>. This is the old name for L</generate1968>.
-It is just kept for backwards compatibility and it calls L</generate>.
+See L<generate|/generate> and L<generate1968|/generate1968>. This is the old name for L<generate1968|/generate1968>.
+It is just kept for backwards compatibility and it calls L<generate|/generate>.
 
 =head2 merge
 
-Mimics L<Hash::Merge>'s L<Hash::Merge/merge> function. Takes two references to
+Mimics L<Hash::Merge|Hash::Merge>'s L<merge|Hash::Merge/merge> function. Takes two references to
 hashes and returns a single reference to a hash containing the merge of the two
-with the left parameter having precendence. The precedence has not meaning on
+with the left parameter having precedence. The precedence has not meaning on
 the case in this module, but then the behaviour is documented.
 
 =head1 PRIVATE FUNCTIONS
 
+=head2 _length
+
+This function validates the length of the argument, it dies if the argument
+does not fit wihtin the boundaries specified by the arguments provided:
+
+The B<_length> function takes the following arguments:
+
+=over
+
+=item number (mandatory), the number to be validated
+
+=item length required of number (mandatory)
+
+=back
+
 =head2 _assertdate
 
 This subroutine takes a digit integer representing a date in the format: DDMMYY.
@@ -500,19 +522,19 @@ Business::DK::CPR exports on request:
 
 =over
 
-=item * L</validate>
+=item * L<validate|/validate>
 
-=item * L</validateCPR>
+=item * L<validateCPR|/validateCPR>
 
-=item * L</validate1968>
+=item * L<validate1968|/validate1968>
 
-=item * L</validate2007>
+=item * L<validate2007|/validate2007>
 
-=item * L</calculate>
+=item * L<calculate|/calculate>
 
-=item * L</generate>
+=item * L<generate|/generate>
 
-=item * L</_checkdate>
+=item * L<_checkdate|/_checkdate>
 
 =back
 
@@ -523,13 +545,13 @@ Business::DK::CPR exports on request:
 =item * 'argument for birthdate should be provided', a data parameter has to be
 provided. 
 
-This error is thrown from L</_checkdate>, which is used for all general parameter
+This error is thrown from L<_checkdate|/_checkdate>, which is used for all general parameter
 validation.
 
 =item * 'argument: <birthdate> could not be parsed', the date provided is not
 represented by 6 digits (see also below).
 
-This error is thrown from L</_checkdate>, which is used for all general parameter
+This error is thrown from L<_checkdate|/_checkdate>, which is used for all general parameter
 validation.
 
 =item * 'argument: <birthdate> has to be a valid date in the format: ddmmyy',
@@ -537,11 +559,11 @@ the date format used for CPR numbers has to adhere to ddmmyy in numeric format
 like so: 311210, day in a two digit representation: 01-31, month also two digit
 representation: 01-12 and finally year in a two digit representation: 00-99.
 
-This error is thrown from L</_checkdate>, which is used for all general parameter
+This error is thrown from L<_checkdate|/_checkdate>, which is used for all general parameter
 validation.
 
 =item * 'Unknown gender: <gender>, assuming no gender', this is just a warning
-issued if a call to L</generate2007> has not been provided with a gender
+issued if a call to L<generate2007|/generate2007> has not been provided with a gender
 parameter
 
 =back
@@ -550,19 +572,17 @@ parameter
 
 =over
 
-=item * L<Business::DK::PO>
+=item * L<Business::DK::CVR|Business::DK::CVR>
 
-=item * L<Business::DK::CVR>
+=item * L<Exporter|Exporter>
 
-=item * L<Exporter>
+=item * L<Carp|Carp>
 
-=item * L<Carp>
+=item * L<Test::Exception|Test::Exception>
 
-=item * L<Test::Exception>
+=item * L<Date::Calc|Date::Calc>
 
-=item * L<Date::Calc>
-
-=item * L<Tie::IxHash>
+=item * L<Tie::IxHash|Tie::IxHash>
 
 =back
 
@@ -578,7 +598,7 @@ There are no known incompatibilies in this package.
 
 =over
 
-=item Nothing to do, please refer to the distribution TODO file for the general
+=item * Nothing to do, please refer to the distribution TODO file for the general
 wish list and ideas for future expansions and experiments.
 
 =back
@@ -589,10 +609,13 @@ The distribution uses the TEST_AUTHOR environment variable to run some
 additional tests, which are interesting to the the author, these can be disabled
 by not defining or setting the environment variable to something not positive.
 
+The distribution uses the TEST_CRITIC environment variable to control
+L<Perl::Critic|Perl::Critic> tests.
+
 =head2 TESTCOVERAGE
 
 Coverage of the test suite is at 89.1% for release 0.04, the coverage report
-was generated with the TEST_AUTHOR flag enabled (SEE: L</TEST AND QUALITY>)
+was generated with the TEST_AUTHOR flag enabled (SEE: L<TEST AND QUALITY|/TEST AND QUALITY>)
 
     ---------------------------- ------ ------ ------ ------ ------ ------ ------
     File                           stmt   bran   cond    sub    pod   time  total
@@ -604,30 +627,33 @@ was generated with the TEST_AUTHOR flag enabled (SEE: L</TEST AND QUALITY>)
 
 =head2 PERL::CRITIC
 
-This section describes use of L<Perl::Critic> from a perspective of documenting
+This section describes use of L<Perl::Critic|Perl::Critic> from a perspective of documenting
 additions and exceptions to the standard use.
 
 =over
 
-=item * L<Perl::Critic::Policy::Miscellanea::ProhibitTies>
+=item * L<Perl::Critic::Policy::Miscellanea::ProhibitTies|Perl::Critic::Policy::Miscellanea::ProhibitTies>
 
-This package utilizes L<Tie::IxHash> (SEE: L</DEPENDENCIES>), this module
+This package utilizes L<Tie::IxHash|Tie::IxHash> (SEE: L<DEPENDENCIES|/DEPENDENCIES>), this module
 relies on tie.
 
-=item * L<Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseSubs>
+=item * L<Perl::Critic::Policy::NamingConventions::NamingConventions::Capitalization|Perl::Critic::Policy::NamingConventions::NamingConventions::Capitalization>
 
-CPR is an abreviation for 'Centrale Person Register' (Central Person Register)
+CPR is an abbreviation for 'Centrale Person Register' (Central Person Register)
 and it is kept in uppercase.
 
-=item * L<Perl::Critic::Policy::ValuesAndExpressions::ProhibitConstantPragma>
+=item * L<Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseSubs|Perl::Critic::Policy::NamingConventions::ProhibitMixedCaseSubs> deprecated by the policy above.
 
-This is a personal thing, but I like constants.
+=item * L<Perl::Critic::Policy::ValuesAndExpressions::ProhibitConstantPragma|Perl::Critic::Policy::ValuesAndExpressions::ProhibitConstantPragma>
 
-=item * L<Perl::Critic::Policy::ValuesAndExpressions::ProhibitMagicNumbers>
+Constants are good in most cases, see also:
+L<http://logicLAB.jira.com/wiki/display/OPEN/Perl-Critic-Policy-ValuesAndExpressions-ProhibitConstantPragma>
+
+=item * L<Perl::Critic::Policy::ValuesAndExpressions::ProhibitMagicNumbers|Perl::Critic::Policy::ValuesAndExpressions::ProhibitMagicNumbers>
 
 Some values and boundaries are defined for certain intervals of numbers, these
 are currently kept as is. Perhaps with a refactoring of the use of constants to
-use of L<Readonly> will address this.
+use of L<Readonly|Readonly> will address this.
 
 =back
 
@@ -642,13 +668,13 @@ to a person is not possible without access to central CPR database an access,
 which is costly, limited and monitored.
 
 There are no other known limitations apart from the obvious flaws in the CPR
-system (See: L</SEE ALSO>).
+system (See: L<SEE ALSO|/SEE ALSO>).
 
 =head1 BUG REPORTING
 
 Please report issues via CPAN RT:
 
-  http://rt.cpan.org/NoAuth/Bugs.html?Dist=Business-DK-CPR
+  L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Business-DK-CPR>
 
 or by sending mail to
 
@@ -660,13 +686,15 @@ or by sending mail to
 
 =item * L<http://www.cpr.dk/>
 
-=item * L<Class::Business::DK::CPR>
+=item * L<Class::Business::DK::CPR|Class::Business::DK::CPR>
 
-=item * L<Data::FormValidator::Constraints::Business::DK::CPR>
+=item * L<Data::FormValidator::Constraints::Business::DK::CPR|Data::FormValidator::Constraints::Business::DK::CPR>
 
-=item * L<Business::DK::PO>
+=item * L<Business::DK::PO|Business::DK::PO>
 
-=item * L<Business::DK::CVR>
+=item * L<Business::DK::CVR|Business::DK::CVR>
+
+=item * L<http://logicLAB.jira.com/wiki/display/OPEN/Perl-Critic-Policy-ValuesAndExpressions-ProhibitConstantPragma>
 
 =back
 
@@ -693,7 +721,7 @@ Jonas B. Nielsen, (jonasbn) - C<< <jonasbn@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Business-DK-CPR is (C) by Jonas B. Nielsen, (jonasbn) 2006-2010
+Business-DK-CPR is (C) by Jonas B. Nielsen, (jonasbn) 2006-2011
 
 =head1 LICENSE
 
@@ -701,6 +729,6 @@ Business-DK-CPR is released under the artistic license
 
 The distribution is licensed under the Artistic License, as specified
 by the Artistic file in the standard perl distribution
-(http://www.perl.com/language/misc/Artistic.html).
+(L<http://www.perl.com/language/misc/Artistic.html>).
 
 =cut
